//
//  FSPagerViewCell.swift
//  FSPagerView
//
//  Created by Wenchao Ding on 17/12/2016.
//  Copyright © 2016 Wenchao Ding. All rights reserved.
//

import UIKit
public protocol FSPagerViewCellDelegate {
    func delete(at: Int)
    func preview(at: Int)
}
open class FSPagerViewCell: UICollectionViewCell {
    var trashButton: UIButton!
    var previewButton: UIButton!
    
    open var delegate: FSPagerViewCellDelegate?
    open var index: Int?
    open var sign: UIImage! {
        didSet {
            loadCaptureSignature(sign ?? UIImage())
        }
    }
    var session: MultiPageScanSession?
    
    private var selectedStickerView:StickerView?
    
    var selectedStickerView1:StickerView? {
        get {
            return selectedStickerView
        }
        set {
            if selectedStickerView != newValue {
                if let selectedStickerView = selectedStickerView {
                    selectedStickerView.showEditing = false
                }
                selectedStickerView = newValue
            }
            if let selectedStickerView = selectedStickerView {
                selectedStickerView.showEditing = true
                selectedStickerView.superview?.bringSubviewToFront(selectedStickerView)
            }
        }
    }
    
    /// Returns the label used for the main textual content of the pager view cell.
    @objc
    open var textLabel: UILabel? {
        if let _ = _textLabel {
            return _textLabel
        }
        let view = UIView(frame: .zero)
        view.isUserInteractionEnabled = false
        view.backgroundColor = UIColor.black.withAlphaComponent(0.6)
        
        let textLabel = UILabel(frame: .zero)
        textLabel.textColor = .white
        textLabel.font = UIFont.preferredFont(forTextStyle: .body)
        self.contentView.addSubview(view)
        view.addSubview(textLabel)
        
        textLabel.addObserver(self, forKeyPath: "font", options: [.old,.new], context: kvoContext)
        
        _textLabel = textLabel
        return textLabel
    }
    
    /// Returns the image view of the pager view cell. Default is nil.
    @objc
    open var imageView: UIImageView? {
        if let _ = _imageView {
            return _imageView
        }
        let imageView = UIImageView(frame: .zero)
        imageView.layer.cornerRadius = 10
        imageView.clipsToBounds = true
        self.contentView.addSubview(imageView)
        _imageView = imageView
        
        let xPosition = 0
        let yPosition = (self.contentView.bounds.height) - 50
        
        trashButton = UIButton(frame: CGRect(x: xPosition, y: Int(yPosition), width: 50, height: 50)) // Adjust frame as needed
        self.contentView.addSubview(trashButton) // Add the button to the cell's contentView
        trashButton.setImage(UIImage(systemName: "trash.fill"), for: .normal)
        trashButton.addTarget(self, action: #selector(topLeftButtonTapped), for: .touchUpInside) // Set the button action
        trashButton.tintColor = .red
        
        
        let previewXPosition = self.contentView.bounds.width - 50
        let previewYPosition = (self.contentView.bounds.height) - 50
        
        previewButton = UIButton(frame: CGRect(x: Int(previewXPosition), y: Int(previewYPosition), width: 50, height: 50)) // Adjust frame as needed
        self.contentView.addSubview(previewButton) // Add the button to the cell's contentView
        previewButton.setImage(UIImage(systemName: "eye"), for: .normal)
        previewButton.addTarget(self, action: #selector(imagePreview), for: .touchUpInside) // Set the button action
        previewButton.tintColor = .red
        
        return imageView
    }
    
    fileprivate weak var _textLabel: UILabel?
    fileprivate weak var _imageView: UIImageView?
    
    fileprivate let kvoContext = UnsafeMutableRawPointer(bitPattern: 0)
    fileprivate let selectionColor = UIColor(white: 0.2, alpha: 0.2)
    
    fileprivate weak var _selectedForegroundView: UIView?
    fileprivate var selectedForegroundView: UIView? {
        guard _selectedForegroundView == nil else {
            return _selectedForegroundView
        }
        guard let imageView = _imageView else {
            return nil
        }
        let view = UIView(frame: imageView.bounds)
        imageView.addSubview(view)
        _selectedForegroundView = view
        return view
    }
    
    open override var isHighlighted: Bool {
        set {
            super.isHighlighted = newValue
            if newValue {
                self.selectedForegroundView?.layer.backgroundColor = self.selectionColor.cgColor
            } else if !super.isSelected {
                self.selectedForegroundView?.layer.backgroundColor = UIColor.clear.cgColor
            }
        }
        get {
            return super.isHighlighted
        }
    }
    

    
    open override var isSelected: Bool {
        set {
            super.isSelected = newValue
            self.selectedForegroundView?.layer.backgroundColor = newValue ? self.selectionColor.cgColor : UIColor.clear.cgColor
        }
        get {
            return super.isSelected
        }
    }
    
    public override init(frame: CGRect) {
        super.init(frame: frame)
        commonInit()
    }
    
    public required init?(coder aDecoder: NSCoder) {
        super.init(coder: aDecoder)
        commonInit()
    }
    
    fileprivate func commonInit() {
        self.contentView.backgroundColor = UIColor.clear
        self.backgroundColor = UIColor.clear
        self.contentView.layer.shadowColor = UIColor.black.cgColor
        self.contentView.layer.shadowRadius = 5
        self.contentView.layer.shadowOpacity = 0.75
        self.contentView.layer.shadowOffset = .zero
        
    }
    
    @objc func topLeftButtonTapped() {
        delegate?.delete(at: index ?? -1)
    }
    
    @objc func imagePreview() {
        delegate?.preview(at: index ?? -1)
    }
    
    deinit {
        if let textLabel = _textLabel {
            textLabel.removeObserver(self, forKeyPath: "font", context: kvoContext)
        }
    }
    
    override open func layoutSubviews() {
        super.layoutSubviews()
        if let imageView = _imageView {
            imageView.frame = self.contentView.bounds
        }
        if let textLabel = _textLabel {
            textLabel.superview!.frame = {
                var rect = self.contentView.bounds
                let height = textLabel.font.pointSize*1.5
                rect.size.height = height
                rect.origin.y = self.contentView.frame.height-height
                return rect
            }()
            textLabel.frame = {
                var rect = textLabel.superview!.bounds
                rect = rect.insetBy(dx: 8, dy: 0)
                rect.size.height -= 1
                rect.origin.y += 1
                return rect
            }()
        }
        if let selectedForegroundView = _selectedForegroundView {
            selectedForegroundView.frame = self.contentView.bounds
        }
    }
    
    open override func observeValue(forKeyPath keyPath: String?, of object: Any?, change: [NSKeyValueChangeKey : Any]?, context: UnsafeMutableRawPointer?) {
        if context == kvoContext {
            if keyPath == "font" {
                self.setNeedsLayout()
            }
        } else {
            super.observeValue(forKeyPath: keyPath, of: object, change: change, context: context)
        }
    }
    
    func grayscale(image: UIImage) -> UIImage?
    {
        let context = CIContext(options: nil)
        if let filter = CIFilter(name: "CIPhotoEffectNoir") {
            filter.setValue(CIImage(image: image), forKey: kCIInputImageKey)
            if let output = filter.outputImage {
                if let cgImage = context.createCGImage(output, from: output.extent) {
                    return UIImage(cgImage: cgImage)
                }
            }
        }
        return nil
    }
    
    func loadCaptureSignature(_ cropImage : UIImage)
    {
        let grayImg = grayscale(image: cropImage)
        let newImage = grayImg
        
        let imageSize = CGSize(width: 125, height: 125)
        
        let testImage = UIImageView()
        testImage.frame.size = imageSize
        testImage.contentMode = .scaleAspectFit
        testImage.image = newImage
        
        let stickerView = StickerView.init(contentView: testImage)
        stickerView.center = contentView.center
        stickerView.delegate = self
        if let exitIcon = UIImage(named: "close"),
           let rotateIcon =  UIImage(named: "resize") {
            stickerView.setImage(exitIcon, forHandler: StickerViewHandler.close)
            stickerView.setImage(rotateIcon, forHandler: StickerViewHandler.rotate)
        }
        
        stickerView.showEditing = false
        
        self.selectedStickerView1 = stickerView
        if let sign = self.selectedStickerView1 {
                contentView.addSubview(sign)
        }
    }
}
extension FSPagerViewCell: StickerViewDelegate {
    func stickerViewDidBeginMoving(_ stickerView: StickerView) {
        return
    }
    
    func stickerViewDidChangeMoving(_ stickerView: StickerView) {
        return
    }
    
    func stickerViewDidEndMoving(_ stickerView: StickerView) {
        return
    }
    
    func stickerViewDidBeginRotating(_ stickerView: StickerView) {
        return
    }
    
    func stickerViewDidChangeRotating(_ stickerView: StickerView) {
        return
    }
    
    func stickerViewDidEndRotating(_ stickerView: StickerView) {
        return
    }
    
    func stickerViewDidClose(_ stickerView: StickerView) {
        return
    }
    
    func stickerViewDidTap(_ stickerView: StickerView) {
        selectedStickerView1?.showEditing = !(selectedStickerView1?.showEditing ?? false)
    }
    
    
}
